@@ -10,17 +10,13 @@ from typing import (
     ContextManager,
     NoReturn,
     Protocol,
+    Self,
     runtime_checkable,
 )
 
 from injection._core.common.asynchronous import AsyncSemaphore, Caller
 from injection._core.common.type import InputType
-from injection._core.scope import (
-    Scope,
-    get_scope,
-    in_scope_cache,
-    remove_scoped_values,
-)
+from injection._core.scope import Scope, get_scope, in_scope_cache
 from injection._core.slots import SlotKey
 from injection.exceptions import EmptySlotError, InjectionError
 
@@ -166,6 +162,10 @@ class ScopedInjectable[R, T](Injectable[T], ABC):
     def __get_scope(self) -> Scope:
         return get_scope(self.scope_name)
 
+    @classmethod
+    def bind_scope_name(cls, name: str) -> Callable[[Caller[..., R]], Self]:
+        return partial(cls, scope_name=name)
+
 
 class AsyncCMScopedInjectable[T](ScopedInjectable[AsyncContextManager[T], T]):
     __slots__ = ()
@@ -199,9 +199,6 @@ class SimpleScopedInjectable[T](ScopedInjectable[T, T]):
     def build(self, scope: Scope) -> T:
         return self.factory.call()
 
-    def unlock(self) -> None:
-        remove_scoped_values(self.key, self.scope_name)
-
 
 @dataclass(repr=False, eq=False, frozen=True, slots=True)
 class ScopedSlotInjectable[T](Injectable[T]):
